﻿local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale("Broker_LFD")
local LFD = CreateFrame("Frame", "Broker_LFD")
local DBversion = "3"
local tipshown

LFD.obj = LibStub("LibDataBroker-1.1"):NewDataObject("Broker_LFD", {type = "data source", icon = "Interface\\AddOns\\Broker_LFD\\Inactive", label = "LFD"})

LFD:RegisterEvent("LFG_UPDATE")
LFD:RegisterEvent("LFG_QUEUE_STATUS_UPDATE")
LFD:RegisterEvent("PLAYER_ENTERING_WORLD")
LFD:RegisterEvent("PARTY_MEMBERS_CHANGED")
LFD:RegisterEvent("LFG_PROPOSAL_SHOW")
LFD:RegisterEvent("LFG_PROPOSAL_UPDATE")
LFD:RegisterEvent("LFG_PROPOSAL_FAILED")
LFD:RegisterEvent("LFG_PROPOSAL_SUCCEEDED")
LFD:RegisterEvent("ADDON_LOADED")

local statusFrame = LFDSearchStatus
local tankWait, healerWait, damageWait, myWait, tankNeeds, healerNeeds, dpsNeeds, queuedTime

LFD.mode = nil 

local function statusChar(char, status, need)
	if status and need and status < need then
		return ("|cff00ff00%s|r"):format(char)
	else
		return ("|cffff0000%s|r"):format(char)
	end
end

local function FormatTime(seconds)
	local time = ""
	local count = 0
	local tempTime
	seconds = floor(seconds)
	maxCount = maxCount or 2
	if ( seconds >= 3600  ) then

		tempTime = floor(seconds / 3600)
		time = time..tempTime.."h"
		seconds = mod(seconds, 3600)
		count = count + 1
	end
	if ( count < maxCount and seconds >= 60  ) then
		tempTime = floor(seconds / 60)
		time = time.." "..tempTime.."m"
		seconds = mod(seconds, 60)
		count = count + 1
	end
	if ( count < maxCount and seconds > 0 and not noSeconds ) then
		time = time.." "..seconds.."s"
	end
	return time
end

local function queueTimeText()
	local text = ""
	local queuedTime = select(13, GetLFGQueueStats())
	if not queuedTime then return text end
	text = ("|cffffffff%s|r"):format(FormatTime(GetTime() - queuedTime))
	return text
end

local function statusText()
	local text = ""
	if not GetLFGMode then return text end
	local tankNeeds, healerNeeds, dpsNeeds = select(3, GetLFGQueueStats())
	text = ("%s%s%s%s%s"):format(
		statusChar(L["T"], tankNeeds, 1),
		statusChar(L["H"], healerNeeds, 1),
		statusChar(L["D"], dpsNeeds, 3),
		statusChar(L["D"], dpsNeeds, 2),
		statusChar(L["D"], dpsNeeds, 1)
	)
	return text
end

LFD:SetScript("OnEvent", function(self, event, arg1)
	--Set up defaults
	if (event == "ADDON_LOADED" and arg1 == "Broker_LFD") and (not Broker_LFDDB or not Broker_LFDDB.dbVersion or Broker_LFDDB.dbVersion < DBversion) then
		LFD:setDefaults()
		return
	end

	local mode, submode = GetLFGMode()
	if mode then
		LFD.obj.icon = "Interface\\AddOns\\Broker_LFD\\Active"
	else
		LFD.obj.icon = "Interface\\AddOns\\Broker_LFD\\Inactive"
	end
	if mode ~= LFD.mode and mode == "queued" then
		LFD.obj.text = ""
	end
	LFD.mode = mode
	statusFrame:SetParent(self)
	statusFrame:SetFrameStrata("TOOLTIP")
	statusFrame:SetClampedToScreen(true)
	MiniMapLFGFrame:Hide()

	if mode == "queued" then
		LFD:SetScript("OnUpdate", function()
			local text = ""
			if Broker_LFDDB.roleStatus then text = text..statusText() end
			if Broker_LFDDB.queueTime then text = text..queueTimeText() end
			LFD.obj.text = text
			if tipshown then LFD.obj.OnEnter() end
		end)
	else
		LFD:SetScript("OnUpdate", nil)
		LFD.obj.text = ""
	end
end)

local function colorFor(wait)
	if wait < 0 then wait = 0 end
	local r, g, b = 0, 0, 0
	if wait > 120 then
		r = 255
		g = (300 - math.min(wait, 300)) / 180 * 255
	else
		g = 255
		r = math.min(wait, 120) / 120 * 255
	end
	return ("%02x%02x%02x"):format(r,g,b)
end

function LFD.obj.OnEnter(self)
	local mode, submode = GetLFGMode()
	tipshown = true
	if ( mode == "queued" ) then
		if Broker_LFDDB.statusFrame then
			statusFrame:ClearAllPoints()
			statusFrame:SetPoint("CENTER", UIParent, "CENTER", Broker_LFDDB.statusXPos, Broker_LFDDB.statusYPos)
			statusFrame:SetScale(Broker_LFDDB.statusScale)
			statusFrame:Show()
		end
		tankWait, healerWait, damageWait, myWait = select(9, GetLFGQueueStats())
		if tankWait then
			local tl = tankWait > 60 and "m" or "s"
			local hl = healerWait > 60 and "m" or "s"
			local dl = damageWait > 60 and "m" or "s"
			local tw = tankWait > 60 and tankWait / 60.0 or tankWait
			local hw = healerWait > 60 and healerWait / 60.0 or healerWait
			local dw = damageWait > 60 and damageWait / 60.0 or damageWait
			GameTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
			GameTooltip:ClearLines()
			GameTooltip:SetText(AVERAGE_WAIT_TIME)
			GameTooltip:AddDoubleLine(TANK, ("|cff%s%2.1f%s|r"):format(colorFor(tankWait), tw, tl))
			GameTooltip:AddDoubleLine(HEALER, ("|cff%s%2.1f%s|r"):format(colorFor(healerWait), hw, hl))
			GameTooltip:AddDoubleLine(DAMAGE, ("|cff%s%2.1f%s|r"):format(colorFor(damageWait), dw, dl))
			GameTooltip:AddDoubleLine(" ")
			GameTooltip:AddDoubleLine(statusText()..queueTimeText())
			GameTooltip:Show()
		end
	elseif ( mode == "proposal" ) then
		GameTooltip:SetText(LOOKING_FOR_DUNGEON)
		GameTooltip:AddLine(DUNGEON_GROUP_FOUND_TOOLTIP, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1)
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(CLICK_HERE_FOR_MORE_INFO, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1)
		GameTooltip:Show()
	elseif ( mode == "rolecheck" ) then
		GameTooltip:SetText(LOOKING_FOR_DUNGEON)
		GameTooltip:AddLine(ROLE_CHECK_IN_PROGRESS_TOOLTIP, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1)
		GameTooltip:Show()
	elseif ( mode == "listed" ) then
		GameTooltip:SetText(LOOKING_FOR_RAID)
		GameTooltip:AddLine(YOU_ARE_LISTED_IN_LFR, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1)
		GameTooltip:Show()
	elseif ( mode == "lfgparty" ) then
		GameTooltip:SetText(LOOKING_FOR_DUNGEON)
		GameTooltip:AddLine(YOU_ARE_IN_DUNGEON_GROUP, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1)
		GameTooltip:Show()
	end
end

function LFD.obj.OnLeave(self)
	GameTooltip:Hide()
	statusFrame:Hide()
	tipshown = nil
end

function LFD.obj.OnClick(self, button)
	local mode, submode = GetLFGMode()
	if ((mode and button == "RightButton") or mode == "lfgparty" or mode == "abandonedInDungeon") then
		statusFrame:Hide()
		MiniMapLFGFrameDropDown.point = "TOPRIGHT"
		MiniMapLFGFrameDropDown.relativePoint = "BOTTOMLEFT"
		ToggleDropDownMenu(1, nil, MiniMapLFGFrameDropDown, "cursor", 0, 0)
	elseif mode == "proposal" then
		if not LFDDungeonReadyPopup:IsShown() then
			PlaySound("igCharacterInfoTab")
			StaticPopupSpecial_Show(LFDDungeonReadyPopup)
		end
	elseif mode == "queued" or mode == "rolecheck" then
		ToggleLFDParentFrame()
	elseif mode == "listed" or (not mode and button == "RightButton") then
		ToggleLFRParentFrame()
	else
		ToggleLFDParentFrame()
	end
end

function LFD:setDefaults()
	Broker_LFDDB = Broker_LFDDB or {}
	Broker_LFDDB.statusFrame = Broker_LFDDB.statusFrame
	Broker_LFDDB.statusScale = Broker_LFDDB.statusScale or 1
	Broker_LFDDB.statusYPos = Broker_LFDDB.statusYPos or 100
	Broker_LFDDB.statusXPos = Broker_LFDDB.statusXPos or 0
	Broker_LFDDB.queueTime = Broker_LFDDB.queueTime
	Broker_LFDDB.rolesStatus = Broker_LFDDB.roleStatus
	Broker_LFDDB.dbVersion = DBversion
end

local options = {
	type ="group",
	name = "Broker LFD",
	handler = BRLFD,
	childGroups = "tree",
	args = {
		header1 = {
			type = "description",
			name = "v1.3",
			order = 0,
			width = "full",
			cmdHidden = true
		},
		spacer1 = {
			type = "header",
			name = "",
			order = 1,
			width = "full",
			cmdHidden = true
		},
		status = {
			type = "group",
			inline = true,
			name = "Status Frame",
			order = 10,
			args = {
				statusFrameEnabled = {
					type = "toggle",
					name = "Enabled",
					desc = "Display the default status frame",
					order = 1,
					set = function (info, value)
						Broker_LFDDB.statusFrame = value
					end,
					get = function() return Broker_LFDDB.statusFrame end
				},
				statusScale = {
					type = "range",
					name = "Scale",
					order = 2,
					min = 0.5,
					max = 11,
					bigStep = 0.1,
					isPercent = true,
					desc = "Adjusts the scale of the default status window.",
					set = function (info, value)
						Broker_LFDDB.statusScale = value
						statusFrame:SetScale(Broker_LFDDB.statusScale)
					end,
					get = function() return Broker_LFDDB.statusScale end
				},
				statusXPos = {
					type = "range",
					name = "Horizontal Position",
					order = 12,
					min = -1000,
					max = 1000,
					bigStep = 10,
					desc = "Adjusts the horizontal of the default status window.",
					set = function (info, value)
						Broker_LFDDB.statusXPos = value
					end,
					get = function() return Broker_LFDDB.statusXPos end
				},
				statusYPos = {
					type = "range",
					name = "Vertical Position",
					order = 13,
					min = -1000,
					max = 1000,
					bigStep = 10,
					desc = "Adjusts the vertical of the default status window.",
					set = function (info, value)
						Broker_LFDDB.statusYPos = value
					end,
					get = function() return Broker_LFDDB.statusYPos end
				},
			},
		},
		other = {
			type = "group",
			inline = true,
			name = "Other",
			order = 20,
			args = {
				rolesStatus = {
					type = "toggle",
					name = "Role Status",
					desc = "Display the roles missing/found on the status text",
					order = 25,
					set = function (info, value)
						Broker_LFDDB.roleStatus = value
						LFD.obj.text = statusText()..queueTimeText()
					end,
					get = function() return Broker_LFDDB.roleStatus end
				},
				queueTime = {
					type = "toggle",
					name = "Queue Time",
					desc = "Display the time you have been queued on the status text",
					order = 27,
					set = function (info, value)
						Broker_LFDDB.queueTime = value
						LFD.obj.text = statusText()..queueTimeText()
					end,
					get = function() return Broker_LFDDB.queueTime end
				},
			},
		},
	},
}

local AceCfg = LibStub("AceConfig-3.0")
local AceCfgReg = LibStub("AceConfigRegistry-3.0")
local AceDlg = LibStub("AceConfigDialog-3.0")

AceCfg:RegisterOptionsTable("Broker_LFD", options)

local brokerOptions = AceCfgReg:GetOptionsTable("Broker", "dialog", "LibDataBroker-1.1")
if (not brokerOptions) then
	brokerOptions = {
		type = "group",
		name = "Broker",
		args = {
		}
	}
	AceCfg:RegisterOptionsTable("Broker", brokerOptions)
	AceDlg:AddToBlizOptions("Broker", "Broker")
end

LFD.optionsFrame = AceDlg:AddToBlizOptions("Broker_LFD", "Looking For Dungeon", "Broker")
