﻿--------------------------------------------------------------------------------------------------------
--                                             Localized global                                       --
--------------------------------------------------------------------------------------------------------
local _G = getfenv(0)

--------------------------------------------------------------------------------------------------------
--                                             AceAddon init                                          --
--------------------------------------------------------------------------------------------------------
local MODNAME = "Broker_TolBarad"
local addon = LibStub("AceAddon-3.0"):NewAddon(MODNAME, "AceEvent-3.0", "AceTimer-3.0")
_G.Broker_TolBarad = addon

local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local L = LibStub("AceLocale-3.0"):GetLocale(MODNAME)
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")

--------------------------------------------------------------------------------------------------------
--                                Broker_TolBarad variables and defaults                              --
--------------------------------------------------------------------------------------------------------
local FACTION_UNKNOWN = 0
local FACTION_ALLIANCE = 1
local FACTION_HORDE = 2
local FACTION_BATTLE = 3

local faction = FACTION_UNKNOWN
local nextBattle = nil

local profileDB
local realmDB
local DATABASE_DEFAULTS = {
	profile = {
		showSeconds = false,
		tellSelf = true,
		tellGuild = true,
		tellParty = false,
		tellRaid = false,
		timeNow = false,
		timeFive = false,
		timeFifteen = true,
		timeThirty = false,
		soundAlert = true,
		autoQueue = false,
	},
	realm = {
		nextBattleTime = nil,
		nextBattleFaction = nil,
	},
}

--------------------------------------------------------------------------------------------------------
--                                 Broker_TolBarad options panel                                      --
--------------------------------------------------------------------------------------------------------
addon.options = {
	type = "group",
	name = "Broker Tol Barad",
	args = {
		general = {
			order = 1,
			type = "group",
			name = L["General Settings"],
			cmdInline = true,
			args = {
				seconds = {
					order = 1,
					type = "toggle",
					name = L["Show seconds"],
					desc = L["Toggle to show seconds."],
					get = function()
						return profileDB.showSeconds
					end,
					set = function(key, value)
						profileDB.showSeconds = value
					end,
				},
				sound = {
					order = 2,
					type = "toggle",
					name = L["Sound alert"],
					desc = L["Toggle to play a sound alert besides the chat warning."],
					get = function()
						return profileDB.soundAlert
					end,
					set = function(key, value)
						profileDB.soundAlert = value
					end,
				},
				queue = {
					order = 3,
					type = "toggle",
					name = L["Auto queue"],
					desc = L["Toggle to automatically atemp to queue 15 minutes before battle."],
					get = function()
						return profileDB.autoQueue
					end,
					set = function(key, value)
						profileDB.autoQueue = value
					end,
				},
				announceTo = {
					order = 2,
					type = 'group',
					name = L["Announce to"],
					desc = L["Choose whom to announce when battle will start."],
					args = {
						self = {
							order = 1,
							type = 'toggle',
							name = L["Self"],
							desc = L["Warn self on upcoming battle."],
							get = function() return profileDB.tellSelf end,
							set = function(key, value) profileDB.tellSelf = value end,
						},
						guild = {
							order = 2,
							type = 'toggle',
							name = L["Guild"],
							desc = L["Warn guild on upcoming battle."],
							get = function() return profileDB.tellGuild end,
							set = function(key, value) profileDB.tellGuild = value end,
						},
						party = {
							order = 3,
							type = 'toggle',
							name = L["Party"],
							desc = L["Warn party on upcoming battle."],
							get = function() return profileDB.tellParty end,
							set = function(key, value) profileDB.tellParty = value end,
						},
						raid = {
							order = 4,
							type = 'toggle',
							name = L["Raid"],
							desc = L["Warn raid on upcoming battle."],
							get = function() return profileDB.tellRaid end,
							set = function(key, value) profileDB.tellRaid = value end,
						},
					},
				},
				announceTime = {
					order = 3,
					type = 'group',
					name = L["Announce time"],
					desc = L["Choose when to announce the upcoming battle."],
					args = {
						now= {
							order = 1,
							type = 'toggle',
							name = L["Start"],
							desc = L["Warn at battle start."],
							get = function() return profileDB.timeNow end,
							set = function(key, value) profileDB.timeNow = value end,
						},
						five= {
							order = 2,
							type = 'toggle',
							name = "5" .. L[" minutes."],
							desc = L["Warn "] .. "5" .. L[" minutes before battle."],
							get = function() return profileDB.timeFive end,
							set = function(key, value) profileDB.timeFive= value end,
						},
						fifteen= {
							order = 3,
							type = 'toggle',
							name = "15" .. L[" minutes."],
							desc = L["Warn "] .. "15" .. L[" minutes before battle."],
							get = function() return profileDB.timeFifteen end,
							set = function(key, value) profileDB.timeFifteen = value end,
						},
						thirty = {
							order = 4,
							type = 'toggle',
							name = "30" .. L[" minutes."],
							desc = L["Warn "] .. "30" .. L[" minutes before battle."],
							get = function() return profileDB.timeThirty end,
							set = function(key, value) profileDB.timeThirty = value end,
						},
					},
				},
			},
		},
	},
}

function addon:SetupOptions()
	addon.options.args.profile = AceDBOptions:GetOptionsTable(self.db)
	addon.options.args.profile.order = -2

	AceConfig:RegisterOptionsTable(MODNAME, Broker_TolBarad.options, nil)
	
	self.optionsFrames = {}
	self.optionsFrames.general = AceConfigDialog:AddToBlizOptions(MODNAME, nil, nil, "general")
	self.optionsFrames.profile = AceConfigDialog:AddToBlizOptions(MODNAME, L["Profiles"], MODNAME, "profile")
end

--------------------------------------------------------------------------------------------------------
--                                          Broker_TolBarad core                                      --
--------------------------------------------------------------------------------------------------------
function addon:OnInitialize()
	self.db = AceDB:New("Broker_TolBaradDB", DATABASE_DEFAULTS, true)
	if not self.db then
		print("Error: Database not loaded correctly.  Please exit out of WoW and delete Broker_TolBarad.lua found in: \\World of Warcraft\\WTF\\Account\\<Account Name>>\\SavedVariables\\")
	end
	
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	
	profileDB = self.db.profile
	realmDB = self.db.realm
	self:SetupOptions()
	
	if (realmDB.nextBattleTime) then
		if (realmDB.nextBattleTime > time()) then
			nextBattle = realmDB.nextBattleTime
			faction = realmDB.nextBattleFaction
		else
			nextBattle = nil
			faction = FACTION_UNKNOWN
		end
		self:UpdateIcon()
	end
end

function addon:OnEnable()
	self:ScheduleRepeatingTimer("MainUpdate", 1)
end

-- LDB object
addon.obj = ldb:NewDataObject(MODNAME, {
	type = "data source",
	label = L["Tol Barad"],
	text = "",
	icon = "Interface\\Icons\\Achievement_win_gilneas",
	OnClick = function(frame, msg)
		if msg == "LeftButton" then
			addon:OnClick()
		elseif msg == "RightButton" then
			addon:ShowConfig()
		end
	end,
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine(MODNAME.." "..GetAddOnMetadata(MODNAME, "Version"))
		tooltip:AddLine(L["Shift-hint"])
		tooltip:AddLine(L["Right-hint"])
	end,
})

-- Main update function
function addon:MainUpdate()
	if faction == FACTION_UNKNOWN or faction == FACTION_BATTLE then
		self:UpdateFaction()
		self:UpdateIcon()
	end
	if nextBattle == nil then
		self:UpdateTime()
	end
	
	local timeText
	if faction == FACTION_BATTLE then
		timeText = L["Battle"]
	elseif not nextBattle then
		timeText = L["Unknown"]
	elseif nextBattle then
		local seconds = nextBattle - time()
		if (seconds > 0) then
			timeText = self:GetTimeText(seconds)
			if (seconds > 8995) then
				faction = FACTION_UNKNOWN
			end
			if seconds == 1800 and profileDB.timeThirty then
				self:Warn(30)
			elseif seconds == 900 and profileDB.timeFifteen then
				self:Warn(15)
			elseif seconds == 300 and profileDB.timeFive then
				self:Warn(5)
			end
			if seconds == 899 and UnitLevel("player") >= 85 and profileDB.autoQueue then
				BattlefieldMgrQueueRequest(21)
			end
		elseif (seconds == 0) then
			if profileDB.timeNow then
				self:Warn(0)
			end
			faction = FACTION_UNKNOWN
			nextBattle = nextBattle + 9000
			timeText = self:GetTimeText(seconds)
		else
			faction = FACTION_UNKNOWN
			nextBattle = nil
			timeText = L["Unknown"]
		end
	end
	if timeText then
		self.obj.text = "|cffffff00" .. timeText .. "|r"
	end
end

--------------------------------------------------------------------------------------------------------
--                                       Broker_TolBarad functions                                    --
--------------------------------------------------------------------------------------------------------

-- Called after profile changed
function addon:OnProfileChanged(event, database, newProfileKey)
	profileDB = database.profile
end

-- Open config window
function addon:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.profile)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.general)
end

-- On click event handler
function addon:OnClick()
	if IsShiftKeyDown() then
		if ChatFrame1EditBox:IsVisible() then
			local message = self:GetAnnounceText()
			ChatFrame1EditBox:Insert(message)
		end
	end
end

-- Update time.
function addon:UpdateTime()
	local pvpID, zone, isActive, canQueue, startTime, canEnter = GetWorldPVPAreaInfo(2);
	if startTime > 0 then 
		nextBattle = time() + startTime
		realmDB.nextBattleTime = nextBattle
	end
end

-- Search for TB status and update faction.
function addon:UpdateFaction()
	local pvpID, zone, isActive, canQueue, startTime, canEnter = GetWorldPVPAreaInfo(2);
	if isActive then
		if not (faction == FACTION_BATTLE) then
			faction = FACTION_BATTLE
		end
	else
		if not WorldMapFrame:IsVisible() and startTime > 0 then
			local found = false
			local continentIndex = GetCurrentMapContinent()
			local zoneIndex = GetCurrentMapZone()
			SetMapZoom(2)
			local n = GetNumMapLandmarks()
			if n > 0 then
				local i = 0
				while (i < n) and (not found) do
					i = i + 1
					local name, _, nr = GetMapLandmarkInfo(i)
					if name == L["Baradin Hold"] then
						found = true
						if nr == 46 then
							faction = FACTION_ALLIANCE
						elseif nr == 48 then
							faction = FACTION_HORDE
						end
					end
				end			
			end
			SetMapZoom(continentIndex, zoneIndex, 1)
			if found then
				self:UpdateTime()
			elseif not (faction == FACTION_UNKNOWN) then
				faction = FACTION_UNKNOWN
			end
		else
			faction = FACTION_UNKNOWN
		end
	end
	realmDB.nextBattleFaction = faction
end

-- Update the icon based on faction.
function addon:UpdateIcon()
	if faction == FACTION_UNKNOWN then
		self.obj.icon = "Interface\\Icons\\Achievement_win_gilneas"
	elseif faction == FACTION_ALLIANCE then
		self.obj.icon = "Interface\\Addons\\Broker_TolBarad\\alliance"
	elseif faction == FACTION_HORDE then
		self.obj.icon = "Interface\\Addons\\Broker_TolBarad\\horde"
	elseif faction == FACTION_BATTLE then
		self.obj.icon = "Interface\\Icons\\Ability_DualWield"
	end
end

-- Return wait time in "h:m:s", or "Battle" if match in progress
function addon:GetTimeText(seconds)
	local timeText = ""
	local h = math.floor(seconds / 3600)
	local m = math.floor((seconds % 3600) / 60)
	local s = seconds % 60
	if profileDB.showSeconds then
		if h > 9 then
			timeText = h .. ":"
		elseif h > 0 then
			timeText = "0" .. h .. ":"
		else
			timeText = "00" .. ":"
		end
		if m > 9 then
			timeText = timeText .. m .. ":"
		elseif m > 0 then
			timeText = timeText .. "0" .. m .. ":"
		else
			timeText = timeText .. "00" .. ":"
		end
		if s > 9 then
			timeText = timeText .. s
		elseif s > 0 then
			timeText = timeText .. "0" .. s
		else
			timeText = timeText .. "00"
		end
	elseif seconds > 59 then
		if h > 9 then
			timeText = h .. ":"
		elseif h > 0 then
			timeText = "0" .. h .. ":"
		else
			timeText = "00" .. ":"
		end
		if m > 9 then
			timeText = timeText .. m
		elseif m > 0 then
			timeText = timeText .. "0" .. m
		else
			timeText = timeText .. "00"
		end
	else
		if m > 9 then
			timeText = timeText .. m .. ":"
		elseif m > 0 then
			timeText = timeText .. "0" .. m .. ":"
		else
			timeText = timeText .. "00" .. ":"
		end
		if s > 9 then
			timeText = timeText .. s
		elseif s > 0 then
			timeText = timeText .. "0" .. s
		else
			timeText = timeText .. "00"
		end
	end
	return timeText
end

function addon:GetBattleTypeText()
	if faction == FACTION_UNKNOWN then
		return L["Next battle in"]
	elseif faction == FACTION_BATTLE then
		return L["The battle for Tol Barad is underway."]
	elseif faction == FACTION_ALLIANCE then
		if "Alliance" == UnitFactionGroup("player") then
			return L["Defensive battle in"]
		else
			return L["Offensive battle in"]
		end
	else
		if "Horde" == UnitFactionGroup("player") then
			return L["Defensive battle in"]
		else
			return L["Offensive battle in"]
		end
	end
end

function addon:GetAnnounceText()
	local message = "[Broker_TolBarad] " .. self:GetBattleTypeText()
	if not (faction == FACTION_BATTLE) then
		if nextBattle then
			message = message .. " " .. self:GetTimeText(nextBattle - time())
		else
			message = message .. " " .. L["Unknown"]
		end
	end
	return message
end

function addon:Warn(minutes)
	local numGuild, numParty, numRaid = GetNumGuildMembers(), GetNumPartyMembers(), GetNumRaidMembers()
	local msg = "[Broker_TolBarad] "
	
	if profileDB.soundAlert then
		PlaySoundFile("Interface\\AddOns\\Broker_TolBarad\\Media\\brass_fanfare_4.mp3");
	end
	
	if minutes == 0 then
		msg = msg .. L["The battle for Tol Barad has begun."]
	else
		msg = msg .. self:GetBattleTypeText() .. " " .. minutes .. L[" minutes."]
	end
	
	if profileDB.tellSelf then
		DEFAULT_CHAT_FRAME:AddMessage("|cffff0000" .. msg .. "|r")
	end
	if profileDB.tellGuild and (numGuild > 0) then
		SendChatMessage(msg,"GUILD",nil,nil)
	end
	if profileDB.tellParty and (numParty > 0) then
		SendChatMessage(msg,"PARTY",nil,nil)
	end
	if profileDB.tellRaid and (numRaid > 0) then
		SendChatMessage(msg,"RAID",nil,nil)
	end
end
