local LibUSC = LibStub and LibStub("LibUserSpecifiedConditions", true)
if not LibUSC then return end

local CATEGORY = "Unit"

LibUSC:RegisterParameter({

    name = "Player",
    version = 1,
    type = "Unit",
    category = CATEGORY,
    value = "player",
})
LibUSC:SetTypeDefault("Unit", "Player")

LibUSC:RegisterParameter({

    name = "Target",
    version = 1,
    type = "Unit",
    category = CATEGORY,
    value = "target",
    Events = {"PLAYER_TARGET_CHANGED"},
    OnEvent = function(self)
        self:FireValueChanged()
    end,
})

LibUSC:RegisterParameter({

    name = "Focus Unit",
    version = 1,
    type = "Unit",
    category = CATEGORY,
    value = "focus",
    Events = {"PLAYER_FOCUS_CHANGED"},
    OnEvent = function(self)
        self:FireValueChanged()
    end,
})

LibUSC:RegisterParameter({

    name = "Pet",
    version = 1,
    type = "Unit",
    category = CATEGORY,
    value = "pet",
})

LibUSC:RegisterParameter({

    name = "Vehicle",
    version = 1,
    type = "Unit",
    category = CATEGORY,
    value = "vehicle",
    Events = {"UNIT_ENTERED_VEHICLE", "UNIT_EXITED_VEHICLE"},
    OnEvent = function(self)
        self:FireValueChanged()
    end,
})

LibUSC:RegisterParameter({

    name = "Mouseover Unit",
    version = 1,
    type = "Unit",
    category = CATEGORY,
    value = "mouseover",
    Events = {"UPDATE_MOUSEOVER_UNIT"},
    OnEvent = function(self)
        self:FireValueChanged()
    end,
})

LibUSC:RegisterParameter({

    name = "Party Member",
    type = "Unit",
    category = CATEGORY,
    version = 1,
    ParameterTypes = {
        "Number",
    },
    configString = "Party Member: {1}",
    
    Call = function(self)
        return "party"..self.Parameters[1].value
    end,
})

LibUSC:RegisterParameter({

    name = "Pet of Party Member",
    type = "Unit",
    category = CATEGORY,
    version = 1,
    ParameterTypes = {
        "Number",
    },
    configString = "Pet of Party Member: {1}",
    
    Call = function(self)
        return "partypet"..self.Parameters[1].value
    end,
})

LibUSC:RegisterParameter({

    name = "Raid Member",
    type = "Unit",
    category = CATEGORY,
    version = 1,
    ParameterTypes = {
        "Number",
    },
    configString = "Raid Member: {1}",
    
    Call = function(self)
        return "raid"..self.Parameters[1].value
    end,
})

LibUSC:RegisterParameter({

    name = "Pet of Raid Member",
    type = "Unit",
    category = CATEGORY,
    version = 1,
    ParameterTypes = {
        "Number",
    },
    configString = "Pet of Raid Member: {1}",
    
    Call = function(self)
        return "raidpet"..self.Parameters[1].value
    end,
})

LibUSC:RegisterParameter({

    name = "Arena Opponent",
    type = "Unit",
    category = CATEGORY,
    version = 1,
    ParameterTypes = {
        "Number",
    },
    configString = "Arena Opponent: {1}",
    
    Call = function(self)
        return "arena"..self.Parameters[1].value
    end,
})

LibUSC:RegisterParameter({

    name = "Boss",
    type = "Unit",
    category = CATEGORY,
    version = 1,
    ParameterTypes = {
        "Number",
    },
    configString = "Boss: {1}",
    
    Call = function(self)
        return "boss"..self.Parameters[1].value
    end,
})

LibUSC:RegisterParameter({

    name = "Target of Unit",
    type = "Unit",
    category = CATEGORY,
    version = 1,
    Events = {"UNIT_TARGET"},
    ParameterTypes = {
        "Unit",
    },
    configString = "Target of {1}",
    
    Call = function(self)
        return self.Parameters[1].value.."target"
    end,
    
    OnEvent = function(self, unit)
        if unit == self.Parameters[1].value then
            self:FireValueChanged()
        end
    end,
})

LibUSC:RegisterParameter({

    name = "Unit Name",
    type = "Text",
    category = CATEGORY,
    version = 1,
    ParameterTypes = {
        "Unit",
    },
    configString = "Name of {1}",
    
    Call = function(self)
        return UnitName(self.Parameters[1].value)
    end,
})

LibUSC:RegisterParameter({

    name = "Unit in Combat",
    version = 1,
    type = "Truth",
    category = CATEGORY,
    Events = {"PLAYER_REGEN_DISABLED", "PLAYER_REGEN_ENABLED"},
    configString = "Player is in Combat",
    
    Call = function(self)
        return InCombatLockdown() == 1
    end,
    
    OnEvent = function(self, event)
        self:SetValue(event == "PLAYER_REGEN_DISABLED")
    end,
})

LibUSC:RegisterParameter({

    name = "Unit is Resting",
    version = 1,
    type = "Truth",
    category = CATEGORY,
    Events = {"PLAYER_UPDATE_RESTING"},
    configString = "Player is Resting",
    
    Call = function(self)
        return IsResting() == 1
    end,
})

LibUSC:RegisterParameter({

    name = "Unit in Vehicle",
    version = 1,
    type = "Truth",
    category = CATEGORY,
    Events = {"UNIT_ENTERED_VEHICLE", "UNIT_EXITED_VEHICLE"},
    ParameterTypes = {
        "Unit",
    },
    configString = "{1} is in a Vehicle",
    
    Call = function(self)
        return UnitInVehicle(self.Parameters[1].value) == 1
    end,
    
    OnEvent = function(self, event, unit)
        if (unit == self.Parameters[1].value) then
            self:SetValue(event == "UNIT_ENTERED_VEHICLE")
        end
    end,
})

LibUSC:RegisterParameter({

    name = "Unit is Casting",
    version = 1,
    type = "Truth",
    category = CATEGORY,
    Events = {
        "UNIT_SPELLCAST_START", 
        "UNIT_SPELLCAST_STOP", 
        "UNIT_SPELLCAST_CHANNEL_START",
        "UNIT_SPELLCAST_CHANNEL_STOP",
    },
    ParameterTypes = {
        "Unit",
    },
    configString = "{1} is Casting",
    
    Call = function(self)
        return (UnitCastingInfo(self.Parameters[1].value) ~= nil)
            or (UnitChannelInfo(self.Parameters[1].value) ~= nil)
    end,
    
    OnEvent = function(self, event, unit)
        if (unit == self.Parameters[1].value) then
            self:SetValue((event == "UNIT_SPELLCAST_START") 
                       or (event == "UNIT_SPELLCAST_CHANNEL_START"))
        end
    end,
})

LibUSC:RegisterParameter({

    name = "Unit Exists",
    version = 1,
    type = "Truth",
    category = CATEGORY,
    ParameterTypes = {
        "Unit",
    },
    configString = "{1} exists",
    
    Call = function(self)
        return UnitExists(self.Parameters[1].value) == 1
    end,
})

LibUSC:RegisterParameter({

    name = "Unit is Unit",
    version = 1,
    type = "Truth",
    category = CATEGORY,
    ParameterTypes = {
        "Unit",
        "Unit",
    },
    configString = "{1} is the same unit as {2}",
    
    Call = function(self)
        return UnitIsUnit(self.Parameters[1].value, self.Parameters[2].value) == 1
    end,
})

LibUSC:RegisterParameter({

    name = "Unit Health",
    type = "Number",
    category = CATEGORY,
    version = 1,
    Events = {"UNIT_HEALTH"},
    ParameterTypes = {
        "Unit",
    },
    configString = "Health of {1}",
    
    Call = function(self)
        return UnitHealth(self.Parameters[1].value)
    end,
    
    OnEvent = function(self, event, unit)
        if (unit == self.Parameters[1].value) then
            self:SetValue()
        end
    end,
})

LibUSC:RegisterParameter({

    name = "Unit Health Maximum",
    type = "Number",
    category = CATEGORY,
    version = 1,
    Events = {"UNIT_MAXHEALTH"},
    ParameterTypes = {
        "Unit",
    },
    configString = "Maximum Health of {1}",
    
    Call = function(self)
        return UnitHealthMax(self.Parameters[1].value)
    end,
    
    OnEvent = function(self, event, unit)
        if (unit == self.Parameters[1].value) then
            self:SetValue()
        end
    end,
})

LibUSC:RegisterParameter({

    name = "Unit Health (Percent)",
    type = "Number",
    category = CATEGORY,
    version = 1,
    Events = {"UNIT_HEALTH", "UNIT_MAXHEALTH"},
    ParameterTypes = {
        "Unit",
    },
    configString = "Percent Health of {1}",
    
    Call = function(self)
        local unit = self.Parameters[1].value
        return UnitHealth(unit)/UnitHealthMax(unit)
    end,
    
    OnEvent = function(self, event, unit)
        if (unit == self.Parameters[1].value) then
            self:SetValue()
        end
    end,
})

LibUSC:RegisterParameter({

    name = "Unit is Moving",
    type = "Truth",
    category = CATEGORY,
    version = 1,
    receivesUpdates = true,
    ParameterTypes = {
        "Unit",
    },
    configString = "{1} is Moving",
    
    Call = function(self)
        return GetUnitSpeed(self.Parameters[1].value) > 0
    end,
})

LibUSC:RegisterParameter({

    name = "Unit Speed",
    type = "Number",
    category = CATEGORY,
    version = 1,
    receivesUpdates = true,
    ParameterTypes = {
        "Unit",
    },
    configString = "Speed of {1}",
    
    Call = function(self)
        return GetUnitSpeed(self.Parameters[1].value)
    end,
})

LibUSC:RegisterParameter({

    name = "Unit Power",
    type = "Number",
    category = CATEGORY,
    version = 1,
    Events = {"UNIT_POWER"},
    ParameterTypes = {
        "Power",
        "Unit",
    },
    configString = "{1} of {2}",
    
    Call = function(self)
        return UnitPower(self.Parameters[2].value, self.Parameters[1].value)
    end,
    
    OnEvent = function(self, event, unit, power)
        if (unit == self.Parameters[2].value) and (power == self.Parameters[1].value) then
            self:SetValue()
        end
    end,
})

LibUSC:RegisterParameter({

    name = "Unit Power Maximum",
    type = "Number",
    category = CATEGORY,
    version = 1,
    Events = {"UNIT_MAXPOWER"},
    ParameterTypes = {
        "Power",
        "Unit",
    },
    configString = "{1} of {2}",
    
    Call = function(self)
        return UnitPowerMax(self.Parameters[2].value, self.Parameters[1].value)
    end,
    
    OnEvent = function(self, event, unit, power)
        if (unit == self.Parameters[2].value) and (power == self.Parameters[1].value) then
            self:SetValue()
        end
    end,
})

LibUSC:RegisterParameter({

    name = "Unit Power (Percent)",
    type = "Number",
    category = CATEGORY,
    version = 1,
    Events = {"UNIT_POWER", "UNIT_MAXPOWER"},
    ParameterTypes = {
        "Power",
        "Unit",
    },
    configString = "{1} of {2}",
    
    Call = function(self)
        local power = self.Parameters[1].value
        local unit = self.Parameters[2].value
        return UnitPower(unit, power)/UnitPowerMax(unit, power)
    end,
    
    OnEvent = function(self, event, unit, power)
        if (unit == self.Parameters[2].value) and (power == self.Parameters[1].value) then
            self:SetValue()
        end
    end,
})
