local Factions = CreateFrame("Frame", "StatBlock_Factions");
Factions.dataObject = LibStub("LibDataBroker-1.1"):NewDataObject("Factions");
local tooltip = LibStub("LibTooltip-1.0");

local GetWatchedFactionInfo = GetWatchedFactionInfo;
local GetItemQualityColor = GetItemQualityColor;
local ToggleCharacter = ToggleCharacter;
local GetNumFactions = GetNumFactions;
local GetFactionInfo = GetFactionInfo;
local ToggleCharacter = ToggleCharacter;
local select = select;
local format = format;
local UIParent = UIParent;
local _G = _G;

local FactionValues = nil;

Factions:RegisterEvent("PLAYER_LOGIN");
Factions:RegisterEvent("UPDATE_FACTION");
Factions:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE");

Factions:SetScript("OnEvent", function(this, event, ...)
    Factions[event](Factions, ...);
end)

local function StandingIdHexColor(standingId)
	local hex;

	if(standingId > 2) then
		hex = select(4, GetItemQualityColor(standingId - 3));
	elseif(standingId == 2) then
		hex = "|cffff0000";
	else
		hex = "|cff8c0000";
	end

	return hex;
end

local Update = function()
	local name, standingId, bottomValue, topValue, earnedValue = GetWatchedFactionInfo();

	if(name) then
		Factions.dataObject.text = format("%s: %s%d/%d|r (%d%%)", name, StandingIdHexColor(standingId), earnedValue - bottomValue, topValue - bottomValue, (earnedValue - bottomValue) / (topValue - bottomValue) * 100);
	else
		Factions.dataObject.text = "Factions"
	end
end

local LoadStartingFactionValues = function()
	FactionValues = { };
	local name, earnedValue, isHeader;
	
	for factionIndex = 1, GetNumFactions() do
		name, _, _, _, _, earnedValue, _, _, isHeader = GetFactionInfo(factionIndex);
		
		if(not isHeader) then
			FactionValues[name] = { startingValue = earnedValue, delta = 0 };
		end
	end
end

function Factions:PLAYER_LOGIN()
	Update();
end

function Factions:UPDATE_FACTION()
	Update()
end

function Factions:CHAT_MSG_COMBAT_FACTION_CHANGE()
	if(not FactionValues) then
		LoadStartingFactionValues();
	end
	
	local name, earnedValue, isHeader;
	
	for factionIndex = 1, GetNumFactions() do		
		name, _, _, _, _, earnedValue, _, _, isHeader = GetFactionInfo(factionIndex);
		
		if(not isHeader) then
			if(not FactionValues[name]) then
				FactionValues[name] = { startingValue = earnedValue, delta = earnedValue };
			else
				FactionValues[name].delta = earnedValue - FactionValues[name].startingValue;
			end
		end
	end
end

function Factions.dataObject.OnClick()
	ToggleCharacter("ReputationFrame");
end

function Factions.dataObject.OnLeave()
	tooltip:Release(Factions.tip);
end

local function GetTipAnchor(frame)
	local x, y = frame:GetCenter();
  
	if(not x or not y) then 
		return "TOP", frame, "BOTTOM" ;
	end
  
	if(y > UIParent:GetHeight() / 2) then
		return "TOP", frame, "BOTTOM";
	else
		return "BOTTOM", frame, "TOP";
	end
end

function Factions.dataObject.OnEnter(frame)	
	local tip = tooltip:Acquire("StatBlock_Factions", 7, "CENTER", "CENTER", "LEFT", "CENTER", "CENTER", "CENTER", "CENTER");
	tip:SetClampedToScreen(true);
	tip:ClearAllPoints();
	tip:SetPoint(GetTipAnchor(frame));
	Factions.tip = tip;
	
	if(not FactionValues) then
		LoadStartingFactionValues();
	end
	
	local lineNum = tip:AddHeader(" ", " ", " ", " ", " ", " ", " ");
	tip:SetCell(lineNum, 1, "|cff00ff00Factions|r", tip.headerFont, "LEFT", 7);
	
	local isHeader, isCollapsed, name, isWatched, standingId, earnedValue, bottomValue, topValue, factionIndex, isChild, delta;
	
	for factionIndex = 1, GetNumFactions() do
		name, _, standingId, bottomValue, topValue, earnedValue, _, _, isHeader, isCollapsed, _, isWatched, isChild = GetFactionInfo(factionIndex);
		
		if(isHeader) then
			if(not isCollapsed) then
				name = format("|cffece909%s|r", name);
			
				if(isChild) then				
					lineNum = tip:AddLine(" ", " ", " ", " ", " ", " ", " ");
					tip:SetCell(lineNum, 1, [[|TInterface\Buttons\UI-MinusButton-Up:0|t]], tip.regularFont, "CENTER");
					tip:SetCell(lineNum, 2, name, tip.regularFont, "LEFT", 6);
				else
					lineNum = tip:AddLine(" ", " ", " ", " ", " ", " ", " ");
					tip:SetCell(lineNum, 1, name, tip.regularFont, "LEFT", 7);
				end
			end
		else
			if(isWatched) then
				isWatched = [[|TInterface\Buttons\UI-CheckBox-Check:0|t]];
			else
				isWatched = " ";
			end
			
			delta = FactionValues[name].delta;
			
			if(delta > 0) then
				delta = format("|cff00ff00+%d|r", delta);
			elseif(delta < 0) then
				delta = format("|cffff0000%d|r", delta);
			else
				delta = " ";
			end
			
			tip:AddLine(" ", isWatched, name, StandingIdHexColor(standingId) .. _G["FACTION_STANDING_LABEL" .. standingId], format("%d/%d", earnedValue - bottomValue, topValue - bottomValue), format("%d%%", (earnedValue - bottomValue) / (topValue - bottomValue) * 100), delta);
		end
	end
		
	tip:Show();
end
