﻿
local Folks = _G.CreateFrame("Frame", "StatBlock_Folks")
Folks:Show()
local L_FRIENDS = _G["FRIENDS"]
Folks.friends = _G.LibStub("LibDataBroker-1.1"):NewDataObject("Folks_Friends", {type = "data source", text = "0 "..L_FRIENDS, value = 0, suffix = L_FRIENDS})

local L_TOOLTIP_F = "\n|cffeda55fClick|r to open the friends panel"
local L_TOOLTIP_G = "\n|cffeda55fClick|r to open the guild panel"

--[[local L = GetLocale()
if L == "frFR" then
	local L_TOOLTIP_F = "\n|cffeda55fClick|r to open the friends panel"
	local L_TOOLTIP_G = "\n|cffeda55fClick|r to open the guild panel"
end]]

local format = string.format
Folks:RegisterEvent("PLAYER_LOGIN")
Folks:SetScript("OnEvent", function(_, event) Folks[event](Folks) end)

function Folks:PLAYER_LOGIN()
	if _G.IsInGuild() then _G.GuildRoster() end
	_G.ShowFriends()
end

local function GetTipAnchor(frame)
  local x,y = frame:GetCenter()
  if not x or not y then return "TOPLEFT", "BOTTOMLEFT" end
  local hhalf = (x > _G.UIParent:GetWidth()*2/3) and "RIGHT" or (x < _G.UIParent:GetWidth()/3) and "LEFT" or ""
  local vhalf = (y > _G.UIParent:GetHeight()/2) and "TOP" or "BOTTOM"
  return vhalf..hhalf, frame, (vhalf == "TOP" and "BOTTOM" or "TOP")..hhalf
end

----------------------------
--        Friends         --
----------------------------
Folks:RegisterEvent("FRIENDLIST_UPDATE")
local Fcount = 0
local fFrame
local GetFriendInfo = _G.GetFriendInfo

function Folks:FRIENDLIST_UPDATE()
	Fcount = 0

	for i = 1, _G.GetNumFriends() do
		local online = select(5, GetFriendInfo(i))
		if online then
			Fcount = Fcount + 1
		end
	end
	self.friends.text = format("%d %s", Fcount, L_FRIENDS)
	self.friends.value = Fcount

	if _G.GameTooltip:IsShown() and _G.GameTooltipTextLeft1:GetText() == L_FRIENDS then
		self.friends.OnEnter(nil, "update")
	end
end

local function RGB2Hex(c)
	return format("%02x%02x%02x", c.r*255, c.g*255, c.b*255)
end

local function Friend2Color(class)
	local result
	for k,v in pairs(_G["LOCALIZED_CLASS_NAMES_FEMALE"]) do
		if v == class then
			result = k
		end
	end
	if not result then
		for k,v in pairs(_G["LOCALIZED_CLASS_NAMES_MALE"]) do
			if v == class then
				result = k
			end
		end
	end
	local c = _G["RAID_CLASS_COLORS"][result]
	return RGB2Hex(c)
end

function Folks.friends.OnEnter(frame, update)
	if frame and not fFrame then
		fFrame = frame
	end
	_G.GameTooltip:SetOwner(fFrame, "ANCHOR_NONE")
	_G.GameTooltip:SetPoint(GetTipAnchor(fFrame))
	_G.GameTooltip:AddDoubleLine(L_FRIENDS, format("[%d/%d]", Fcount, _G.GetNumFriends()), 0, 1, 1, 1, 1, 0.5)
	if not update then
		_G.GameTooltip:Show()
		_G.ShowFriends()
		return
	end

	for i = 1, Fcount do
		local name, level, class, zone, _, status, note = GetFriendInfo(i)
		local difficultyColor = RGB2Hex(_G.GetQuestDifficultyColor(level))
		local classColor = Friend2Color(class)
		local groupIndicator = ""
		local zoneColor = " |cffffffff"

		if _G.UnitInParty(name) or _G.UnitInRaid(name) then
			groupIndicator = "|cff0fff0f+|r"
		end

		if not note then note = "" end

		if note ~= "" then
			note = " [|cffffffff"..note.."|r]"
		end

		if zone and zone == _G.GetRealZoneText() then
			zoneColor = " |cff0fff0f"
		else
			zoneColor = " |cffffffff"
		end

		_G.GameTooltip:AddDoubleLine(
			format("%s|cff%s%d|r |cff%s%s|r%s", groupIndicator, difficultyColor, level, classColor, name, note or ""), 
			status..zoneColor..zone or _G["UNKNOWN"].."|r"
		)
	end
	_G.GameTooltip:AddLine(L_TOOLTIP_F)
	_G.GameTooltip:Show()
end

function Folks.friends.OnLeave()
	_G.GameTooltip:Hide()
end

function Folks.friends.OnClick()
	_G.ToggleFriendsFrame(1) -- Friends
end

----------------------------
--         Guild          --
----------------------------

local GetGuildRosterInfo = _G.GetGuildRosterInfo
local sorted = nil
local gUpdate = nil
local gFrame
local Gcount = 0

local L_GUILD = _G["GUILD"]
Folks.guild = _G.LibStub("LibDataBroker-1.1"):NewDataObject("Folks_Guild", {type = "data source", text = "0 "..L_GUILD, value = 0, suffix = L_GUILD})

Folks:RegisterEvent("PLAYER_GUILD_UPDATE")
Folks:RegisterEvent("GUILD_ROSTER_UPDATE")
Folks:RegisterEvent("CHAT_MSG_SYSTEM")

function Folks:GUILD_ROSTER_UPDATE()
	if not _G.IsInGuild() then return end
	Gcount = 0

	if gUpdate then
		gUpdate = nil
		Folks:SetScript("OnUpdate", nil)
	end

	if _G.GetGuildRosterShowOffline() then
		for i = 1, _G.GetNumGuildMembers() do
			local online = select(9, GetGuildRosterInfo(i))
			if online then
				Gcount = Gcount + 1
			end
		end
	else
		Gcount = _G.GetNumGuildMembers()
	end
	self.guild.text = format("%d %s", Gcount, L_GUILD)
	self.guild.value = Gcount

	if _G.GameTooltip:IsShown() and _G.GameTooltipTextLeft1:GetText() == L_GUILD then
		self.guild.OnEnter()
	end
end

Folks.PLAYER_GUILD_UPDATE = Folks.GUILD_ROSTER_UPDATE

--Short timer incase we can't do GuildRoster() when the event is triggered (10 sec CD)
function Folks:CHAT_MSG_SYSTEM()
	_G.ShowFriends()
	gUpdate = true
	local prev = 0
	local GetTime = _G.GetTime
	Folks:SetScript("OnUpdate", function(_, e)
		local time = GetTime()
		if (time - prev) < 1 then return end
		prev = time
		if not _G.IsInGuild() then
			Folks:SetScript("OnUpdate", nil)
			Gcount = 0
			self.guild.text = format("%d %s", Gcount, L_GUILD)
			self.guild.value = Gcount
			return
		end
		_G.GuildRoster()
	end)
end

local function Guild2Color(class)
	local c = _G["RAID_CLASS_COLORS"][class]
	return RGB2Hex(c)
end

function Folks.guild.OnEnter(frame)
	if not _G.IsInGuild() then return end
	if frame and not gFrame then
		gFrame = frame
	end

	_G.GameTooltip:SetOwner(gFrame, "ANCHOR_NONE")
	_G.GameTooltip:SetPoint(GetTipAnchor(gFrame))
	local Gtotal = _G.GetNumGuildMembers(true)
	_G.GameTooltip:AddDoubleLine(L_GUILD, format("[%d/%d]", Gcount, Gtotal), 0, 1, 1, 1, 1, 0.5)

	if not sorted then
		sorted = true
		_G.GameTooltip:Show()
		_G.SortGuildRoster("Name")
		return
	end
	_G.GuildRoster()

	_G.GameTooltip:AddLine(_G.GetGuildRosterMOTD().."\n", 1, 1, 1, true)

	if _G.GetGuildRosterShowOffline() then Gcount = Gtotal end
	for i = 1, Gcount do
		local name, rank, _, level, _, zone, note, officernote, online, status, trueClass = GetGuildRosterInfo(i)
		if online then
			local difficultyColor = RGB2Hex(_G.GetQuestDifficultyColor(level))
			local guildRankNoteText = " [|cffffffff"..rank.."|r]"
			local classColor = Guild2Color(trueClass)
			local groupIndicator = ""
			local zoneColor = " |cffffffff"

			if _G.UnitInParty(name) or _G.UnitInRaid(name) then
				groupIndicator = "|cff0fff0f+|r"
			end

			guildRankNoteText = guildRankNoteText.."[|cffffffff"..note.."|r]"

			if _G.CanViewOfficerNote() then
				guildRankNoteText = guildRankNoteText.."[|cffffffff"..officernote.."|r]"
			end

			if zone and zone == _G.GetRealZoneText() then
				zoneColor = " |cff0fff0f"
			else
				zoneColor = " |cffffffff"
			end

			_G.GameTooltip:AddDoubleLine(
				format("%s|cff%s%d|r |cff%s%s|r%s", groupIndicator, difficultyColor, level, classColor, name, guildRankNoteText),
				status..zoneColor..(zone or _G["UNKNOWN"]).."|r"
			)
		end
	end
	_G.GameTooltip:AddLine(L_TOOLTIP_G)
	_G.GameTooltip:Show()
end

function Folks.guild.OnLeave()
	_G.GameTooltip:Hide()
end

function Folks.guild.OnClick()
	_G.ToggleFriendsFrame(3) -- Guild
end

